﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.BinaryVectorEncoding.SolutionModel {
  [Item("Solution Sampling Operator", "Creates new solutions by sampling from a given model.")]
  [StorableClass]
  public class SolutionSamplingOperator : InstrumentedOperator {
    
    public ILookupParameter<UnivariateModel> ModelParameter {
      get { return (ILookupParameter<UnivariateModel>)Parameters["Model"]; }
    }

    public IValueLookupParameter<IntValue> NumberOfSampledSolutionsParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["NumberOfSampledSolutions"]; }
    }

    public ILookupParameter<BinaryVector> BinaryVectorParameter {
      get { return (ILookupParameter<BinaryVector>)Parameters["BinaryVector"]; }
    }

    [StorableConstructor]
    protected SolutionSamplingOperator(bool deserializing) : base() { }
    protected SolutionSamplingOperator(SolutionSamplingOperator original, Cloner cloner)
      : base(original, cloner) { }
    public SolutionSamplingOperator() {
      Parameters.Add(new LookupParameter<UnivariateModel>("Model", "The model that is being trained by this operator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberOfSampledSolutions", "The amount of solutions that should be created through sampling."));
      Parameters.Add(new LookupParameter<BinaryVector>("BinaryVector", "The vector which should be manipulated."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SolutionSamplingOperator(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var model = ModelParameter.ActualValue;
      var samples = NumberOfSampledSolutionsParameter.ActualValue.Value;
      var solutionName = BinaryVectorParameter.TranslatedName;
      var scope = ExecutionContext.Scope;
      for (var i = 0; i < samples; i++) {
        var solution = model.Sample();
        var solutionScope = new Scope(scope.SubScopes.Count.ToString());
        solutionScope.Variables.Add(new Variable(solutionName, solution));
        scope.SubScopes.Add(solutionScope);
      }
      return base.InstrumentedApply();
    }
  }
}
